/*
-05/10/2006: Petre M.
  -this project represents a Reference Design for a power meter using ADE7169Fxx
*/


#include "main.h"

  void INT0_ISR(void) interrupt 0 
{
}

  void TIMER0_ISR(void) interrupt 1
{
}

  void INT1_ISR(void) interrupt 2
{
}

  void TIMER1_ISR(void) interrupt 3
{
}

  void UART_ISR(void) interrupt 4
{
}

  void TIMER2_ISR(void) interrupt 5
{
}

  void TEMPERATURE_ISR(void) interrupt 6
{
}

  void SPI_I2C_ISR(void) interrupt 7
{
}

  void PSM_ISR(void) interrupt 8
{
}

  void ADE_ISR(void) interrupt 9
{
  ADE_Interrupt();
}

  void RTC_ISR(void) interrupt 10
{


}

  void WATCHDOG_ISR(void) interrupt 11
{
}

  void main( void )
{
  void (* LP_func_ptr)(void);
//P0 ^=Bit3;//toggle CF2

  //if reset provoked by watchdog, turn the display full on and blink it
  Test_Watchdog();

  //Initializations common to both PSM0 and PSM1 modes

  //disable all interrupt sources before doing the initializations
  EA = 0;

  Setup_Watchdog(WD_1sec);//sets PRE=6 -> 1sec for watchdog period


  Setup_I2C();

  //LCD SFRs are initialized and the display is erased
  Setup_LCD();

  Setup_Power();

  Init_variables();


  //if VSW=VDD, then the part functions in PSM0 mode
  if (PERIPH & VSWSOURCE) {
    //put PSM0 initializations here

    Setup_Power_PSM0();

    Enable_Pull_Ups();

//!!!    Setup_UART();

    Init_variables_PSM0();//initializes some of the variables to avoid having the compiler doing it

    //enable all interrupt sources
    EA = 1;

    Setup_RTC();

    //we need to initialize all the ADE registers. Some of them use values from EEPROM
    Setup_Energy();

    //read from EEPROM all data previously stored in EEPROM
    //The program waits until the data is stored in IRAM!!!!
    Reload_from_EEPROM();

    //at power up, IPSMF_bit.FVDC may stay in 1 for a certain period of time.
    //This would provoke execution of Backup_State() function. We need to
    //introduce a time before the regular PSM0 loop is executed.
    Wait_loop();
    Wait_loop();

    //if VSW=VDD, then the part functions in PSM0 mode
    //This is the PSM0 loop
    while (PERIPH & VSWSOURCE) {


      //put all High Priority tasks here
      Backup_State();//this function executes a backup of Energy Meter state if VDCIN<1.2V
      Accumulate_Energy();
//!!      Manage_UART_Rx();//this function manages UART communication with the PC

     if (TIMECON & ALARM) {
        //clear ALARM flag
        TIMECON = TIMECON & (~ALARM);

        //set Bit0 in Low_Priority_Command[0]
        Set_flag(Bit0, &Low_Priority_Command[0]);
     }

      //this code below ensures LP_Task stays always between 0 and sizeof(Table_LP_Func)/2)-1
      if (Low_Priority_Command[0] & Bit0) {
        if (LP_Task>(sizeof(Table_LP_Func)/2)-1) {
          LP_Task = 0;
        }
        if (LP_Task<0) {
          LP_Task = 0;
        }

        //execute one Low Priority Task
        LP_func_ptr = (void (*) (void))Table_LP_Func[LP_Task];
        (*LP_func_ptr)();

        //increment pointer
        LP_Task=LP_Task+1;

      }
    }
  }


 
  //disable Energy Metering Interrupt
  IEIP2 &= (~EADE);

  //disable all periodical background measurements
  STRBPER = 0x00;

  //we now execute PSM1 initializations
  Setup_RTC_PSM1();

  //in PSM1 mode, blink the LCD at 0.25Hz
  Blink_LCD(Blink_250mHz);

  Disable_Pull_Ups();

  Init_variables_PSM1();

  //enable all interrupt sources only after all variables have been initialized
  EA = 1;

  //Now, the program executes PSM1 tasks

  Manage_Display();
  Display_to_LCD();
  Midnight_Management_PSM1();
  Monthly_Energy_Storage();
  Prepare_for_Sleep();
  Refresh_Watchdog();//this function prevents watchdog from timeout

  //enter PSM2 mode (Deep Sleep): Bit4 of POWCON=1
  //set P0.6=SCLK and P0.4=SDATA with weak pull up enabled otherwise the processor still
  //consumes power in sleep mode
  EA = 0;//disable all interrupts
  KYREG = 0xa7;
  POWCON=0x10;

  //in case the processor has changed supply from the battery to VDD,
  //the command to enter PSM2 does not have any effect. For this reason, this instruction
  //is executed only if the supply has changed to VDD
  //The opcode of this instruction is 0x020000
//#pragma asm
//    ljmp 0x0;
//#pragma endasm
	((void (code *) (void)) 0x0000) ();   
}






